<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Services\CadastroIntegracaoService;
use Teknisa\MAN\Services\ParamKDSService;
use Teknisa\MAN\Services\AppsIntegrations\CallJenkins\IIntegracaoJenkinsJobs;
use Teknisa\MAN\Utilities;

class CadastroIntegracao extends \Teknisa\MAN\Controllers\TeknisaCrud {

    const dataSourceIntegracoes            = 'GetIntegracoesRepository';
    const dataSourceRecebimentos           = 'GetIntegracaoRepositoryRecebimentos';
    const dataSourceRecebimentosToken      = 'GetIntegracaoRepositoryRecebimentosToken';
    const dataSourceRecebimentoDisponivel  = 'RecebimentosDisponiveisRepo';
    const dataSourceUnidades               = 'GetIntegracaoRepositoryUnidades';
    const dataSourceTokens                 = 'GetUnidadeTokens';
    const dataSourceTipoRecebimentosIntegracao  = 'RecebimentosDisponiveisIntegracaoRepo';
    const dataSourceTipoRecebimentosToken  = 'RecebimentosDisponiveisTokenRepo';
    const dataSourceLJCX                   = 'selectIntegracaoLJCX';

    protected $parameterBagColumnsDefault = array('NRORG', 'CDINTEGRACAO');
    protected $parameterBagUnidades = array('NRORG');
    protected $parameterBagRecebimentoDiponivel = array('NRORG','CDTIPORECE');
    protected $parameterBagTokens = array('NRORG', 'CDFILIAL', 'CDINTEGRACAO');
    protected $parameterBagRecebimentos = array('NRORG', 'CDINTEGRACAO','CDTPRECEINTE');
    protected $parameterBagTipoRecebimentos = array('NRORG');
    protected $parameterBagRecebimentosToken = array('NRORG', 'CDINTEGRACAO','CDTPRECEINTE', 'CDFILIAL', 'NMFILIAL');
    protected $parameterBagTipoRecebimentosToken = array('NRORG', 'CDINTEGRACAO');
    protected $parameterBagTipoRecebimentosIntegracao = array('NRORG', 'CDINTEGRACAO');
    protected $parameterBagLJCX = array('CDFILIAL', 'CDINTEGRACAO', 'NRORG');

    private $jenkinsAPIService;

    public function __construct (
        $crudService,
        $parameterBag,
        $paramBag,
        Manager $dataSourceManager,
        Utilities\Util $util,
        CadastroIntegracaoService $cadastroIntService,
        IIntegracaoJenkinsJobs $jenkinsAPIService) {
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->dataSourceManager        = $dataSourceManager;
        $this->util                     = $util;
        $this->cadService               = $cadastroIntService;
        $this->jenkinsAPIService        = $jenkinsAPIService;
    }

    public function findRecebimentosDisponiveis(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentoDisponivel;
        $this->parameterBagColumns = $this->parameterBagRecebimentoDiponivel;
        parent::find($request,$response);
    }

    public function findRecebimentosDisponiveisIntegracao(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceTipoRecebimentosIntegracao;
        $this->parameterBagColumns = $this->parameterBagTipoRecebimentosIntegracao;
        parent::find($request,$response);
    }

    public function findRecebimentosDisponiveisToken(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceTipoRecebimentosToken;
        $this->parameterBagColumns = $this->parameterBagTipoRecebimentosToken;
        parent::find($request,$response);
    }

    public function saveIntegracao(DTO\Request $request, DTO\Response $response){
        $params = $request->getDataSet()->getRows();

        if($params[0]['__is_new'])
            $params[0]['CDINTEGRACAO'] = $this->util->newCode('INTEGRACAO', '3');

        $this->dataSourceName = self::dataSourceIntegracoes;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::save($request,$response);
    }

    public function saveRecebimentos(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentos;
        $params = $request->getDataSet()->getRows();

        /* Deleta para depois salvar */
        if(!$params[0]['__is_new']) {
            $cdTipoReceChanged = $params[0]['CDTIPORECE'];
            $cdTpReceInte = $params[0]['CDTPRECEINTE'];

            $params[0]['CDTIPORECE'] = $params[0]['CDTIPORECE_BK'];
            $params[0]['CDTPRECEINTE'] = $params[0]['CDTPRECEINTE_BK'];
            $params[0]['NMTIPORECE'] = $params[0]['NMTIPORECE_BK'];

            $this->deleteDataSetWithoutNotification($request, $response);

            $params[0]['CDTIPORECE'] = $cdTipoReceChanged;
            $params[0]['CDTPRECEINTE'] = $cdTpReceInte;

            $params[0]['__is_new'] = true;
        }
        
        $this->parameterBagColumns = $this->parameterBagRecebimentos;
        parent::save($request,$response);
    }

    public function saveRecebimentosToken(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentosToken;
        $params = $request->getDataSet()->getRows();

        /* Deleta para depois salvar */
        if(!$params[0]['__is_new']) {
            $cdTipoReceChanged = $params[0]['CDTIPORECE'];
            $cdTpReceInte = $params[0]['CDTPRECEINTE'];

            $params[0]['CDTIPORECE'] = $params[0]['CDTIPORECE_BK'];
            $params[0]['CDTPRECEINTE'] = $params[0]['CDTPRECEINTE_BK'];
            $params[0]['NMTIPORECE'] = $params[0]['NMTIPORECE_BK'];

            $this->deleteDataSetWithoutNotification($request, $response);

            $params[0]['CDTIPORECE'] = $cdTipoReceChanged;
            $params[0]['CDTPRECEINTE'] = $cdTpReceInte;

            $params[0]['__is_new'] = true;
        }
        
        $this->parameterBagColumns = $this->parameterBagRecebimentosToken;
        parent::save($request,$response);
    }

    public function removeIntegracao(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceIntegracoes;
        self::deleteRowWithoutNotification($request,$response);
    }

    public function getIntegracoes(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceIntegracoes;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request,$response);
    }

    public function findRecebimentos(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentos;
        $this->parameterBagColumns = $this->parameterBagRecebimentos;
        parent::find($request,$response);
    }

    public function findRecebimentosToken(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentosToken;
        $this->parameterBagColumns = array('NRORG', 'CDINTEGRACAO', 'CDFILIAL');
        parent::find($request,$response);
    }

    public function removeRecebimentos(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentos;
        self::deleteDataSetWithoutNotification($request,$response);
    }

    public function removeRecebimentosToken(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentosToken;
        self::deleteDataSetWithoutNotification($request,$response);
    }

    public function deleteDataSetWithoutNotification($request, $response) {
		try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}

	public function deleteRowWithoutNotification(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findUnidades(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceUnidades;
        $this->parameterBagColumns = $this->parameterBagUnidades;
        parent::find($request,$response);
    }

    public function findTokens(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceTokens;
        $this->parameterBagColumns = $this->parameterBagTokens;
        parent::find($request,$response);
    }

    public function saveTokens(DTO\Request $request, DTO\Response $response){
        $params = $this->util->getParamsAsArray($request);
        $unique =  $this->cadService->searchToken($params[0]);
        if($unique && $params[0]['__is_new']){
            return $response->setError(new DTO\Response\Error('Token Informado já existe na banco de dados!', ''));
        }
        if(!$params[0]['__is_new']) {
            try{
                $params = $this->util->getParamsAsArray($request);
                if(!\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDLOJA', $params[0])){
                    $params[0]['CDLOJA'] = null;
                }
                $retorno = $this->cadService->updateToken($params[0]);
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }else {
            try{
                $params = $this->util->getParamsAsArray($request);
                if(!\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDLOJA', $params[0])){
                    $params[0]['CDLOJA'] = null;
                }
                $retorno = $this->cadService->addToken($params[0]);
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function removeTokens(DTO\Request\Row $request, DTO\Response $response){
        $params = $this->util->getParamsAsArray($request);

        try{
            $params = $this->util->getParamsAsArray($request);
            $retorno = $this->cadService->removeToken($params);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function startJenkinsJob(DTO\Request\Row $request, DTO\Response $response) {
        $params = $this->util->getParamsAsArray($request);

        $IDINTEGRACAO = $params['IDINTEGRACAO'];

        try {
            $result = $this->jenkinsAPIService->startJenkinsJob($IDINTEGRACAO);
            $response->addNotification(new Notification("O serviço foi iniciado com sucesso!", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function stopJenkinsJob(DTO\Request\Row $request, DTO\Response $response) {
        $params = $this->util->getParamsAsArray($request);

        $IDINTEGRACAO = $params['IDINTEGRACAO'];

        try {
            $result = $this->jenkinsAPIService->stopJenkinsJob($IDINTEGRACAO);
            $response->addNotification(new Notification("O serviço foi iniciado com sucesso!", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findIntegracaoLJCX(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceLJCX;
        $this->parameterBagColumns = $this->parameterBagLJCX;
        parent::find($request,$response);
    }
    
    public function saveIntegracaoLJCX(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName(self::dataSourceLJCX);
            $rows = $dataSet->getRows();
            unset($rows[0]['CDCAIXA']);
            unset($rows[0]['CDLOJA']);
            
            foreach($rows[0]['CDLJCAIXA'] as $loja){
                $aux = explode('_', $loja);
                $rows[0]['CDLOJA'] = $aux[0];
                $rows[0]['CDCAIXA'] = $aux[1];
                $validacao = $this->cadService->getLJCXRepetidas($rows[0]);
                
                if(count($validacao) > 0) {
                    throw new \Exception("Caixa já cadastrado!");
                } else {
                    $persistedRows = $this->dataSourceManager->persist($dataSet);
                    $response->addDataSet(new DataSet(self::dataSourceLJCX));
                    $nrPersistedRows = count($persistedRows);
                }
            }
            
        } catch (Exception $e) {
            $response->addDataSet(new DataSet('error', [$e->getMessage()]));
        }
    }
    
    public function removeIntegracaoLJCX(DTO\Request\DataSet $request,DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName(self::dataSourceLJCX);
            $rows = $dataSet->getRows();
            foreach($rows as $row){
                $this->cadService->removeLJCXRepetidas($row);
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}