<?php

namespace Teknisa\MAN\Services;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\NewCode;
use Teknisa\MAN\Utilities;


class ParamOdhenGoService {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function saveGeralTab($conditions){
        try{

            $conditions = $conditions[0];
            $senhaSalva = $this->entityManager->getConnection()->fetchAll(self::FIND_SENHA_SHA512);

            //Se a senha enviada pelo frontend for a máscara usada por segurança, a senha atual(salva no banco) deve ser a usada.
            if($conditions['CDSENHAOGOSHA512'] == 'seNHAbaCKupDeSegurANCA321'){
                $params = array(
                    'CDSENHAOGOSHA512'  => $senhaSalva[0]['CDSENHAOGOSHA512'],
                    'CDCLIENTE'         => $conditions['CDCLIENTE'],
                    'NRORG'             => $conditions['NRORG']
                );

            } else {
                $params = array(
                    'CDSENHAOGOSHA512'  => hash('sha512', $conditions['CDSENHAOGOSHA512']),
                    'CDCLIENTE'         => $conditions['CDCLIENTE'],
                    'NRORG'             => $conditions['NRORG']
                );

            }
            $requestParams = array(
                'NRORG' => $params['NRORG'],
                'HASH'  => $params['CDSENHAOGOSHA512']
            );
            $endpoint = 'http://briansilva1.zeedhi.com/workfolder/odhengo-aws/cadastrarhashorganizacao';
            $hash = self::makeAwsRequest($requestParams, $endpoint);

            $update = $this->entityManager->getConnection()->executeQuery(self::UPDATE_PARAM_GERAL, $params);
            $retorno = array (
                'HASH'      => $hash,
                'UPDATE'    => $update
            );
            return $retorno;

        } catch(Exception $e){
            throw $e;
        }
    }

    public function saveFilialTab($conditions){
        try {
            $conditions = $conditions[0];
            $params = array (
                'CDFILIAL'  => $conditions['CDFILIAL'],
                'NRORG'     => $conditions['NRORG']
            );
            $verificacao = $this->entityManager->getConnection()->fetchAll(self::VERIFY_FILIAL_ROWS, $params);
            unset($params);

            if($verificacao[0]['REGISTROS'] > 0){
                $params = array(
                    'CDFILIAL'              => $conditions['CDFILIAL'],
                    'NRORG'                 => $conditions['NRORG'],
                    'IDTPIDENTIFICACONS'    => $conditions['IDTPIDENTIFICACONS']
                );
                return $this->entityManager->getConnection()->executeQuery(self::UPDATE_PARAM_FILIAL, $params);
            } else {
                $params = array(
                    'CDFILIAL'              => $conditions['CDFILIAL'],
                    'NRORG'                 => $conditions['NRORG'],
                    'IDTPIDENTIFICACONS'    => $conditions['IDTPIDENTIFICACONS'],
                    'DTINCLUSAO'            => date('d/m/Y')
                );
                return $this->entityManager->getConnection()->executeQuery(self::INSERT_PARAM_FILIAL, $params);
            }

        } catch (Exception $e){
            throw $e;
        }
    }

    public function savePdvTab($conditions){
        try{
            $conditions = $conditions[0];
            $params = array(
                'CDFILIAL'      => $conditions['CDFILIAL'],
                'CDCAIXA'       => $conditions['CDCAIXA'],
                'CDLOJA'        => $conditions['CDLOJA'],
                'NRORG'         => $conditions['NRORG'],
                'IDCXATIVOOGO'  => $conditions['IDCXATIVOOGO']

            );
            return $this->entityManager->getConnection()->executeQuery(self::UPDATE_STATUS_PDV, $params);
        } catch (Exception $e){
            throw $e;
        }
    }

    public function findCapturesAWS($conditions){
        try{
            $senhaSalva = $this->entityManager->getConnection()->fetchAll(self::FIND_SENHA_SHA512);
            $dataAtual = date('d/m/Y');
            $requestParams = array(
                'NRORG'     => $conditions['NRORG'],
                'CDFILIAL'  => $conditions['CDFILIAL'],
                'CDCAIXA'   => $conditions['CDCAIXA'],
                'HASH'      => $senhaSalva[0]['CDSENHAOGOSHA512'],
                'DTIMAGENS' => date('d/m/Y')
            );
            $endpoint = 'http://briansilva1.zeedhi.com/workfolder/odhengo-aws/imagemvenda/listar';
            $capturas = self::makeAwsRequest($requestParams, $endpoint);
            $retorno = array();

            if(isset($capturas->status) && $capturas->status){
                foreach($capturas->datas->$dataAtual as $imagem){
                    $image = array (
                        'DTENTRVENDA' => $dataAtual,
                        'IMAGE'       => $imagem
                    );
                    array_push($retorno, $image);
                }
            }

            return $retorno;
        } catch (Exception $e){
            throw $e;
        }
    }

    public function findFilesAWS($conditions){
        try {
            $senhaSalva = $this->entityManager->getConnection()->fetchAll(self::FIND_SENHA_SHA512);
            $requestParams = array(
                'NRORG' => $conditions['NRORG'],
                'HASH'  => $senhaSalva[0]['CDSENHAOGOSHA512']
            );

            $endpoint = 'http://briansilva1.zeedhi.com/workfolder/odhengo-aws/listararquivos';
            $files = self::makeAwsRequest($requestParams, $endpoint);
            $retorno = array();

            if($files){
                foreach($files->dados as $arquivo){
                    $arqDataset = array(
                        'CDARQDATASET'      => $arquivo
                    );
                    array_push($retorno, $arqDataset);
                }
            }

            return $retorno;

        } catch (Exception $e){
            throw $e;
        }

    }

    public function getDatasetDownloadLink($conditions){
        try{
            $senhaSalva = $this->entityManager->getConnection()->fetchAll(self::FIND_SENHA_SHA512);
            $requestParams = array(
                'NRORG'     => $conditions['NRORG'],
                'FILENAME'  => $conditions['CDARQDATASET'],
                'HASH'      => $senhaSalva[0]['CDSENHAOGOSHA512']
            );
            $endpoint = "http://briansilva1.zeedhi.com/workfolder/odhengo-aws/baixararquivo";
            $link = self::makeAwsRequest($requestParams, $endpoint);

            $retorno = array(
                'STATUS'    => false,
                'LINK'      => ''
            );

            if($link->status){
                $retorno["STATUS"] = $link->status;
                $retorno["LINK"] = $link->link;
            }

            return $retorno;
        } catch (Exception $e){
            throw $e;
        }
    }

    private function makeAwsRequest($requestParams, $endpoint) {
        try{
            $cr = curl_init();
            curl_setopt($cr, CURLOPT_URL, $endpoint);
            curl_setopt($cr, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($cr, CURLOPT_HTTPPROXYTUNNEL, 0);
            curl_setopt($cr, CURLOPT_POSTFIELDS, $requestParams);
            curl_setopt($cr, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
            curl_setopt($cr, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($cr, CURLOPT_TIMEOUT, 30);
            $return = json_decode(curl_exec($cr));
            return $return;
        } catch(Exception $e){
            throw $e;
        }
    }

    const UPDATE_PARAM_GERAL = "
        UPDATE PARAMGERAL
        SET CDCLIENTE = :CDCLIENTE,
            CDSENHAOGOSHA512 = :CDSENHAOGOSHA512
        WHERE NRORG = :NRORG
    ";

    const VERIFY_FILIAL_ROWS = "
        SELECT COUNT(*) AS REGISTROS FROM PARAMODHGO
        WHERE CDFILIAL = :CDFILIAL
          AND NRORG = :NRORG
    ";

    const UPDATE_PARAM_FILIAL = "
        UPDATE PARAMODHGO
        SET IDTPIDENTIFICACONS = :IDTPIDENTIFICACONS
        WHERE CDFILIAL = :CDFILIAL
          AND NRORG = :NRORG
    ";

    const INSERT_PARAM_FILIAL = "
        INSERT INTO PARAMODHGO
          (CDFILIAL, IDTPIDENTIFICACONS, NRORG, IDATIVO, DTINCLUSAO)
        VALUES
          (:CDFILIAL, :IDTPIDENTIFICACONS, :NRORG, 'S', :DTINCLUSAO)
    ";

    const UPDATE_STATUS_PDV = "
        UPDATE CAIXA
        SET IDCXATIVOOGO = :IDCXATIVOOGO
        WHERE CDFILIAL = :CDFILIAL
          AND CDCAIXA = :CDCAIXA
          AND CDLOJA = :CDLOJA
          AND NRORG = :NRORG
    ";

    const FIND_SENHA_SHA512  = "
        SELECT CDSENHAOGOSHA512 FROM PARAMGERAL
    ";
}